package org.ugasp.impl.games.demonstrator.test;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.ugasp.impl.games.demonstrator.protocol.moods.FirstMooDSObject;
import org.ugasp.impl.games.demonstrator.protocol.rmai.objects.MyRMAIObject;
import org.ugasp.protocol.rmai.RMAIGameLogicManager;


/**
 * This class holds the methods that will be invoked remotely by other clients 
 * or by game logic.
 * 
 * Declare methods respecting following properties:
 * <ul>
 * <li>they must be instances methods, and not static ones</li>
 * <li>types handled as parameters are Byte, Short, Integer, Long, Float, Double, Boolean, String, Date and Calendar</li>
 * <li>as method invocations through RMAI protocol are asynchronous,
 * only void methods can be invoked. However a callback mechanism is provided, allowing to invoke remote method
 * on the entity which did the invocation first (as it is done in replyToMessage(String message))</li>
 * </ul>
 * 
 * Entity which has invoked a method located in this class can be retrieved, using getSenderActorSessionID().
 * 
 * @see replyToMessage(String message)
 **/
public class RMAIGameLogicManagerImpl extends RMAIGameLogicManager{

	private GameLogicDemonstratorForTests gameLogic;
	// added for test
	private static File gameLogicTestFile;
	private static DataOutputStream gameLogicTestFileDos = null;
	
	private final static MyRMAIObject rmaiSerializable0 = initRMAISerializable0();
	private final static MyRMAIObject rmaiSerializable1 = initRMAISerializable1();
	
	
	public RMAIGameLogicManagerImpl(GameLogicDemonstratorForTests gameLogic, short senderActorSessionID, short recipientActorSessionID){
		super(senderActorSessionID, recipientActorSessionID);
		this.gameLogic = gameLogic;
	}
	
	private final static MyRMAIObject initRMAISerializable0(){
		return new MyRMAIObject("Bodmer", "PSG", Short.valueOf((short)25), Byte.valueOf((byte)8));		
	}
	
	private final static MyRMAIObject initRMAISerializable1(){
		return new MyRMAIObject("Sakho", "PSG", Short.valueOf((short)21), Byte.valueOf((byte)4));
	}
	
	
	/*********************************************************************************************************/
	/********************************    Method used to test MooDS    ****************************************/
	/*********************************************************************************************************/

	
	public static void storeMooDSMessage(String mooDSMessage) throws IOException{
		if(gameLogicTestFileDos != null){
			// writing value on file for later comparison
			gameLogicTestFileDos.writeUTF(mooDSMessage);
		}
	}
	
	
	public void gameLogicToClientMooDS(){
		FirstMooDSObject fmo = new FirstMooDSObject();
		fmo.setText(GameLogicDemonstratorForTests.STRING_TO_TEST_MooDS);
		// here, the "sender" of the RMAI is the recipient of the message to send
		gameLogic.sendMooDSMessageToPlayer(fmo, ActorSessionID.uGASP_GAME_LOGIC.getID(),senderActorSessionID);
	}
	
	
	/*********************************************************************************************************/
	/*************************************    Remote Methods    **********************************************/
	/*********************************************************************************************************/
	
	public void prepareFileWriting(String filePath){
		gameLogicTestFile = new File(filePath);
		try{
			// get stream from gameLogicTestFile
			FileOutputStream gameLogicTestFileOutputStream = new FileOutputStream(gameLogicTestFile);
			gameLogicTestFileDos = new DataOutputStream(gameLogicTestFileOutputStream);
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}
	}
	
	
	public void askToQuitTest(){
		closeDataOutputStream();
		// invoking this method on the client side make it call quitApplicationInstance()
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID, "quitTest");
	}
	
	
	private void closeDataOutputStream(){
		try{
			if(gameLogicTestFileDos != null){
				gameLogicTestFileDos.flush();
				gameLogicTestFileDos.close();
			}
		}catch(IOException e){
			e.printStackTrace();
		}
	}
	
	/**********************************************************************************************/
	/**************************  Method used to test RMAI on the game logic side ******************/
	/**********************************************************************************************/
	
	public void testBytes(Byte b0, Byte b1, Byte b2) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeByte(b0);
		gameLogicTestFileDos.writeByte(b1);
		gameLogicTestFileDos.writeByte(b2);
	}
	
	
	public void testShorts(Short s0, Short s1, Short s2) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeShort(s0);
		gameLogicTestFileDos.writeShort(s1);
		gameLogicTestFileDos.writeShort(s2);
	}
	
	
	public void testIntegers(Integer i0, Integer i1, Integer i2) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeInt(i0);
		gameLogicTestFileDos.writeInt(i1);
		gameLogicTestFileDos.writeInt(i2);
	}
	
	
	public void testLongs(Long l0, Long l1, Long l2) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeLong(l0);
		gameLogicTestFileDos.writeLong(l1);
		gameLogicTestFileDos.writeLong(l2);
	}
	
	
	public void testFloats(Float f0, Float f1, Float f2, Float f3, Float f4, Float f5) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeFloat(f0);
		gameLogicTestFileDos.writeFloat(f1);
		gameLogicTestFileDos.writeFloat(f2);
		gameLogicTestFileDos.writeFloat(f3);
		gameLogicTestFileDos.writeFloat(f4);
		gameLogicTestFileDos.writeFloat(f5);
	}
	
	
	public void testDoubles(Double d0, Double d1, Double d2, Double d3, Double d4, Double d5) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeDouble(d0);
		gameLogicTestFileDos.writeDouble(d1);
		gameLogicTestFileDos.writeDouble(d2);
		gameLogicTestFileDos.writeDouble(d3);
		gameLogicTestFileDos.writeDouble(d4);
		gameLogicTestFileDos.writeDouble(d5);
	}
	
	
	public void testBooleans(Boolean b0, Boolean b1) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeBoolean(b0);
		gameLogicTestFileDos.writeBoolean(b1);
	}
	
	
	public void testStrings(String s0, String s1) throws IOException{
		// writing values on file for later comparison
		gameLogicTestFileDos.writeUTF(s0);
		gameLogicTestFileDos.writeUTF(s1);
	}
	
	
	public void testSerializables(MyRMAIObject fro1,MyRMAIObject fro2) throws IOException{
		// writing String representations of FirstRMAIObjects for later comparison
		gameLogicTestFileDos.writeUTF(fro1.toString());
		gameLogicTestFileDos.writeUTF(fro2.toString());
	}
	
	/**********************************************************************************************/
	/**************************  Method used to test RMAI on the client side **********************/
	/**********************************************************************************************/

	/**
	 * This RMAI asks client to create the data output stream used for tests.
	 * Once its done, client sends back a RMAI to game logic in order to make it launch test
	 * via a call to gameLogicToClientRMAI()
	 */
	public void askToCreateDataOutputStream(){
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,"createDataOutputStream");
		
	}
	
	public void gameLogicToClientTests(){
		// tests on RMAI
		gameLogicToClientRMAI();
		// test on MooDS
		gameLogicToClientMooDS();
		// store data for later comparison
		storeDataInTestFile();
		//rmai to quit tests
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"quitTest");
	}
	
	public void gameLogicToClientRMAI(){
		// rmai to test bytes
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testBytes",
				Byte.MIN_VALUE,new Byte((byte)0),Byte.MAX_VALUE);
		// rmai to test shorts
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testShorts",
				Short.MIN_VALUE,new Short((short)0), Short.MAX_VALUE);
		// rmai to test integers
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testIntegers",
				Integer.MIN_VALUE,new Integer((int)0), Integer.MAX_VALUE);
		// rmai to test longs
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testLongs",
				Long.MIN_VALUE,new Long((long)0), Long.MAX_VALUE);
		// rmai to test floats
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testFloats",
				Float.NEGATIVE_INFINITY,Float.MIN_VALUE,
				new Float((float)0), Float.NaN,
				Float.MAX_VALUE,Float.POSITIVE_INFINITY);
		// rmai to test doubles
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testDoubles",
				Double.NEGATIVE_INFINITY,Double.MIN_VALUE,
				new Double((double)0), Double.NaN,
				Double.MAX_VALUE,Double.POSITIVE_INFINITY);
		// rmai to test booleans
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testBooleans",
				Boolean.TRUE,Boolean.FALSE);
		// rmai to test Strings	
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testStrings",
				"AZERTY","azerty");
		// rmai to test RMAISerializable	
		gameLogic.remoteMethodAsynchronousInvocation(senderActorSessionID,
				"testSerializables",rmaiSerializable0,rmaiSerializable1);
	}
	
	
	/*
	 * This method store the data sent to game logic in a file to allow a later comparison
	 * with data received by game logic.
	 */
	private void storeDataInTestFile(){
		try{
			// writing values on file for later comparison
			// bytes
			gameLogicTestFileDos.writeByte(Byte.MIN_VALUE);
			gameLogicTestFileDos.writeByte(new Byte((byte)0));
			gameLogicTestFileDos.writeByte(Byte.MAX_VALUE);
			// shorts
			gameLogicTestFileDos.writeShort(Short.MIN_VALUE);
			gameLogicTestFileDos.writeShort(new Short((short)0));
			gameLogicTestFileDos.writeShort(Short.MAX_VALUE);
			// integers
			gameLogicTestFileDos.writeInt(Integer.MIN_VALUE);
			gameLogicTestFileDos.writeInt(new Integer(0));
			gameLogicTestFileDos.writeInt(Integer.MAX_VALUE);
			// longs
			gameLogicTestFileDos.writeLong(Long.MIN_VALUE);
			gameLogicTestFileDos.writeLong(new Long((long)0));
			gameLogicTestFileDos.writeLong(Long.MAX_VALUE);
			// floats
			gameLogicTestFileDos.writeFloat(Float.NEGATIVE_INFINITY);
			gameLogicTestFileDos.writeFloat(Float.MIN_VALUE);
			gameLogicTestFileDos.writeFloat(new Float((float)0));
			gameLogicTestFileDos.writeFloat(Float.NaN);
			gameLogicTestFileDos.writeFloat(Float.MAX_VALUE);
			gameLogicTestFileDos.writeFloat(Float.POSITIVE_INFINITY);
			// doubles
			gameLogicTestFileDos.writeDouble(Double.NEGATIVE_INFINITY);
			gameLogicTestFileDos.writeDouble(Double.MIN_VALUE);
			gameLogicTestFileDos.writeDouble(new Double((double)0));
			gameLogicTestFileDos.writeDouble(Double.NaN);
			gameLogicTestFileDos.writeDouble(Double.MAX_VALUE);
			gameLogicTestFileDos.writeDouble(Double.POSITIVE_INFINITY);
			// booleans
			gameLogicTestFileDos.writeBoolean(Boolean.TRUE);
			gameLogicTestFileDos.writeBoolean(Boolean.FALSE);
			// Strings
			gameLogicTestFileDos.writeUTF("AZERTY");
			gameLogicTestFileDos.writeUTF("azerty");
			// RMAISerializables
			gameLogicTestFileDos.writeUTF(rmaiSerializable0.toString());
			gameLogicTestFileDos.writeUTF(rmaiSerializable1.toString());
			// MooDS message 
			gameLogicTestFileDos.writeUTF(GameLogicDemonstratorForTests.STRING_TO_TEST_MooDS);
			// close streams
			gameLogicTestFileDos.flush();
			gameLogicTestFileDos.close();    
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}catch(IOException e){
			e.printStackTrace();
		}       
	}
	
	
	/******************************************************************************************************/
	/**************************  Method used to test broadcast RMAI on the game logic side ******************/
	/******************************************************************************************************/

	
	public void storeBroadcastWithGameLogicMessage(String message){
		if( gameLogicTestFileDos != null){
			try {
				gameLogicTestFileDos.writeUTF(message);
				gameLogicTestFileDos.close();
			}catch (IOException e) {
				e.printStackTrace();
			}
		} else{
			System.out.println("Game logic test file output stream NULL, can't write on it");
		}
	}
	

}
